@@ -53,11 +53,10 @@
 #include "fetchmail.h"
 #include "tunable.h"
 
-RETSIGTYPE
+static RETSIGTYPE
 sigchld_handler (int sig)
 /* process SIGCHLD to obtain the exit code of the terminating process */
 {
-    extern volatile int lastsig;		/* last signal received */
     pid_t pid;
 
 #if 	defined(HAVE_WAITPID)				/* the POSIX way */
@@ -260,7 +259,7 @@ nottyDetach:
   return(0);
 }
 
-flag isafile(int fd)
+flag is_a_file(int fd)
 /* is the given fd attached to a file? (used to control logging) */
 {
     struct stat stbuf;
