@@ -25,7 +25,7 @@ static int etrn_ok (int sock, char *argbuf)
 {
     int ok;
 
-    ok = SMTP_ok(sock);
+    ok = SMTP_ok(sock, SMTP_MODE);
     if (ok == SM_UNRECOVERABLE)
 	return(PS_PROTOCOL);
     else
@@ -40,7 +40,7 @@ static int etrn_getrange(int sock, struct query *ctl, const char *id,
     char buf [MSGBUFSIZE+1];
     struct idlist *qnp;		/* pointer to Q names */
 
-    if ((ok = SMTP_ehlo(sock, fetchmailhost, 
+    if ((ok = SMTP_ehlo(sock, SMTP_MODE, fetchmailhost,
 			ctl->server.esmtp_name, ctl->server.esmtp_password,
 			&opts)))
     {
@@ -65,7 +65,7 @@ static int etrn_getrange(int sock, struct query *ctl, const char *id,
     for (qnp = ctl->domainlist; qnp; qnp = qnp->next)
     {
 	/* ship the actual poll and get the response */
-	gen_send(sock, "ETRN %s", qnp->id);
+	gen_send(sock, "ETRN %s", (char *)qnp->id);
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
 	    return(ok);
 
@@ -119,28 +119,25 @@ static int etrn_logout(int sock, struct query *ctl)
     return(gen_transact(sock, "QUIT"));
 }
 
-const static struct method etrn =
+static const struct method etrn =
 {
     "ETRN",		/* ESMTP ETRN extension */
-#if INET6_ENABLE
     "smtp",		/* standard SMTP port */
     "smtps",		/* ssl SMTP port */
-#else /* INET6_ENABLE */
-    25,			/* standard SMTP port */
-    465,			/* ssl SMTP port */
-#endif /* INET6_ENABLE */
     FALSE,		/* this is not a tagged protocol */
     FALSE,		/* this does not use a message delimiter */
     etrn_ok,		/* parse command response */
     NULL,		/* no need to get authentication */
     etrn_getrange,	/* initialize message sending */
     NULL,		/* we cannot get a list of sizes */
+    NULL,		/* we cannot get a list of sizes of subsets */
     NULL,		/* how do we tell a message is old? */
     NULL,		/* no way to fetch headers */
     NULL,		/* no way to fetch body */
     NULL,		/* no message trailer */
     NULL,		/* how to delete a message */
     NULL,		/* how to mark a message as seen */
+    NULL,		/* no mailbox support */
     etrn_logout,	/* log out, we're done */
     FALSE,		/* no, we can't re-poll */
 };
@@ -159,7 +156,7 @@ int doETRN (struct query *ctl)
 	return(PS_SYNTAX);
     }
     if (ctl->mailboxes->id) {
-	fprintf(stderr, GT_("Option --remote is not supported with ETRN\n"));
+	fprintf(stderr, GT_("Option --folder is not supported with ETRN\n"));
 	return(PS_SYNTAX);
     }
     if (check_only) {
