@@ -24,10 +24,12 @@ MIT license.  Compile with -DMAIN to build the demonstrator.
 #include  <string.h>
 #include  <stdlib.h>
 
-#ifndef MAIN
 #include "fetchmail.h"
+
+#ifndef MAIN
 #include "i18n.h"
 #else
+#include  <unistd.h>
 static int verbose;
 char *program_name = "rfc822";
 #endif /* MAIN */
@@ -39,11 +41,11 @@ char *program_name = "rfc822";
 
 #define HEADER_END(p)	((p)[0] == '\n' && ((p)[1] != ' ' && (p)[1] != '\t'))
 
-unsigned char *reply_hack(buf, host, length)
+unsigned char *reply_hack(
+	unsigned char *buf		/* header to be hacked */,
+	const unsigned char *host	/* server hostname */,
+	size_t *length)
 /* hack message headers so replies will work properly */
-unsigned char *buf;		/* header to be hacked */
-const unsigned char *host;	/* server hostname */
-int *length;
 {
     unsigned char *from, *cp, last_nws = '\0', *parens_from = NULL;
     int parendepth, state, has_bare_name_part, has_host_part;
@@ -93,7 +95,7 @@ int *length;
 #ifdef MAIN
 	if (verbose)
 	{
-	    printf("state %d: %s", state, buf);
+	    printf("state %d: %s", state, (char *)buf);
 	    printf("%*s^\n", from - buf + 10, " ");
 	}
 #endif /* MAIN */
@@ -213,9 +215,8 @@ int *length;
     return(buf);
 }
 
-unsigned char *nxtaddr(hdr)
+unsigned char *nxtaddr(const unsigned char *hdr /* header to be parsed, NUL to continue previous hdr */)
 /* parse addresses in succession out of a specified RFC822 header */
-const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 {
     static unsigned char address[BUFSIZ];
     static int tp;
@@ -251,7 +252,7 @@ const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr *
 #ifdef MAIN
 	if (verbose)
 	{
-	    printf("state %d: %s", state, orighdr);
+	    printf("state %d: %s", state, (char *)orighdr);
 	    printf("%*s^\n", hp - orighdr + 10, " ");
 	}
 #endif /* MAIN */
@@ -263,7 +264,7 @@ const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr *
 	    state = ENDIT_ALL;
 	    if (tp)
 	    {
-		while (isspace(address[--tp]))
+		while (tp >= 0 && isspace(address[tp--]))
 		    continue;
 		address[++tp] = '\0';
 		tp = 0;
@@ -344,13 +345,9 @@ const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr *
 	    break;
 
 	case INSIDE_DQUOTE:	/* we're in a quoted string, copy verbatim */
-	    if (*hp != '"')
-	        address[NEXTTP()] = *hp;
-	    else
-	    {
-	        address[NEXTTP()] = *hp;
+	    address[NEXTTP()] = *hp;
+	    if (*hp == '"')
 		state = oldstate;
-	    }
 	    break;
 
 	case INSIDE_PARENS:	/* we're in a parenthesized comment, ignore */
@@ -392,23 +389,24 @@ const unsigned char *hdr;	/* header to be parsed, NUL to continue previous hdr *
 static void parsebuf(unsigned char *longbuf, int reply)
 {
     unsigned char	*cp;
+    size_t		dummy;
 
     if (reply)
     {
-	reply_hack(longbuf, "HOSTNAME.NET");
-	printf("Rewritten buffer: %s", longbuf);
+	reply_hack(longbuf, "HOSTNAME.NET", &dummy);
+	printf("Rewritten buffer: %s", (char *)longbuf);
     }
     else
 	if ((cp = nxtaddr(longbuf)) != (unsigned char *)NULL)
 	    do {
-		printf("\t-> \"%s\"\n", cp);
+		printf("\t-> \"%s\"\n", (char *)cp);
 	    } while
 		((cp = nxtaddr((unsigned char *)NULL)) != (unsigned char *)NULL);
 }
 
 
 
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     unsigned char	buf[BUFSIZ], longbuf[BUFSIZ];
     int			ch, reply;
@@ -429,13 +427,13 @@ main(int argc, char *argv[])
     while (fgets(buf, sizeof(buf)-1, stdin))
     {
 	if (buf[0] == ' ' || buf[0] == '\t')
-	    strcat(longbuf, buf);
+	    strlcat(longbuf, buf, sizeof(longbuf));
 	else if (!strncasecmp("From: ", buf, 6)
 		    || !strncasecmp("To: ", buf, 4)
 		    || !strncasecmp("Reply-", buf, 6)
 		    || !strncasecmp("Cc: ", buf, 4)
 		    || !strncasecmp("Bcc: ", buf, 5))
-	    strcpy(longbuf, buf);	
+	    strlcpy(longbuf, buf, sizeof(longbuf));
 	else if (longbuf[0])
 	{
 	    if (verbose)
@@ -450,6 +448,7 @@ main(int argc, char *argv[])
 	    fputs(longbuf, stdout);
 	parsebuf(longbuf, reply);
     }
+    exit(0);
 }
 #endif /* MAIN */
 
