@@ -14,6 +14,7 @@
 #include <memory.h>
 #endif /* HAVE_MEMORY_H */
 #include <sys/types.h>
+#include <sys/stat.h>
 #ifndef HAVE_NET_SOCKET_H
 #include <sys/socket.h>
 #else
@@ -36,8 +37,20 @@
 #else
 #include <varargs.h>
 #endif
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include "socket.h"
 #include "fetchmail.h"
+#include "getaddrinfo.h"
 #include "i18n.h"
 
 /* Defines to allow BeOS and Cygwin to play nice... */
@@ -72,14 +85,8 @@ static int h_errno;
 
 #endif /* ndef h_errno */
 
-extern int mailserver_socket_temp;	/* Socket to close if connect timeout */
-
-#if NET_SECURITY
-#include <net/security.h>
-#endif /* NET_SECURITY */
-
 #ifdef HAVE_SOCKETPAIR
-char *const *parse_plugin(const char *plugin, const char *host, const char *service)
+static char *const *parse_plugin(const char *plugin, const char *host, const char *service)
 {	const char **argvec;
 	const char *c, *p;
 	char *cp, *plugin_copy;
@@ -91,7 +98,7 @@ char *const *parse_plugin(const char *plugin, const char *host, const char *serv
 	unsigned int service_len = strlen(service);
 
 	for (c = p = plugin; *c; c++)
-	{	if (isspace(*c) && !isspace(*p))
+	{	if (isspace((unsigned char)*c) && !isspace((unsigned char)*p))
 			s += sizeof(char*);
 		if (*p == '%' && *c == 'h')
 			host_count++;
@@ -135,14 +142,14 @@ char *const *parse_plugin(const char *plugin, const char *host, const char *serv
 	}
 	memset(argvec, 0, s);
 	for (c = p = plugin_copy, i = 0; *c; c++)
-	{	if ((!isspace(*c)) && (c == p ? 1 : isspace(*p))) {
+	{	if ((!isspace((unsigned char)*c)) && (c == p ? 1 : isspace((unsigned char)*p))) {
 			argvec[i] = c;
 			i++;
 		}
 		p = c;
 	}
 	for (cp = plugin_copy; *cp; cp++)
-	{	if (isspace(*cp))
+	{	if (isspace((unsigned char)*cp))
 			*cp = 0;
 	}
 	return (char *const*)argvec;
@@ -170,7 +177,6 @@ static int handle_plugin(const char *host,
 		/* error */
 		report(stderr, GT_("fetchmail: fork failed\n"));
 		return -1;
-		break;
 	case 0:	/* child */
 		/* fds[1] is the parent's end; close it for proper EOF
 		** detection */
@@ -199,7 +205,6 @@ static int handle_plugin(const char *host,
 #endif /* HAVE_SOCKETPAIR */
 
 #ifdef __UNUSED__
-#include <sys/time.h>
 
 int SockCheckOpen(int fd)
 /* poll given socket; is it selectable? */
@@ -258,16 +263,11 @@ int UnixOpen(const char *path)
     return sock;
 }
 
-#if INET6_ENABLE
-int SockOpen(const char *host, const char *service, const char *options,
+int SockOpen(const char *host, const char *service,
 	     const char *plugin)
 {
     struct addrinfo *ai, *ai0, req;
     int i;
-#if NET_SECURITY
-    void *request = NULL;
-    int requestlen;
-#endif /* NET_SECURITY */
 
 #ifdef HAVE_SOCKETPAIR
     if (plugin)
@@ -276,30 +276,15 @@ int SockOpen(const char *host, const char *service, const char *options,
     memset(&req, 0, sizeof(struct addrinfo));
     req.ai_socktype = SOCK_STREAM;
 
-    if (getaddrinfo(host, service, &req, &ai0)) {
-	report(stderr, GT_("fetchmail: getaddrinfo(%s.%s)\n"), host,service);
+    i = getaddrinfo(host, service, &req, &ai0);
+    if (i) {
+	report(stderr, GT_("getaddrinfo(\"%s\",\"%s\") error: %s\n"),
+		host, service, gai_strerror(i));
+	if (i == EAI_SERVICE)
+	    report(stderr, GT_("Try adding the --service option (see also FAQ item R12).\n"));
 	return -1;
     }
 
-#if NET_SECURITY
-    if (!options)
-	requestlen = 0;
-    else
-	if (net_security_strtorequest((char *)options, &request, &requestlen))
-	    goto ret;
-
-    i = inner_connect(ai0, request, requestlen, NULL, NULL, "fetchmail", NULL);
-    if (request)
-	free(request);
-
- ret:
-#else /* NET_SECURITY */
-#ifdef HAVE_INNER_CONNECT
-    i = inner_connect(ai0, NULL, 0, NULL, NULL, "fetchmail", NULL);
-    if (i >= 0)
-	break;
-#else
-
     i = -1;
     for (ai = ai0; ai; ai = ai->ai_next) {
 	i = socket(ai->ai_family, ai->ai_socktype, 0);
@@ -323,153 +308,10 @@ int SockOpen(const char *host, const char *service, const char *options,
 	break;
     }
 
-#endif
-#endif /* NET_SECURITY */
-
     freeaddrinfo(ai0);
 
     return i;
 }
-#else /* INET6_ENABLE */
-#ifndef HAVE_INET_ATON
-#ifndef  INADDR_NONE
-#ifdef   INADDR_BROADCAST
-#define  INADDR_NONE	INADDR_BROADCAST
-#else
-#define	 INADDR_NONE	-1
-#endif
-#endif
-#endif /* HAVE_INET_ATON */
-
-int SockOpen(const char *host, int clientPort, const char *options,
-	     const char *plugin)
-{
-    int sock = -1;	/* pacify -Wall */
-#ifndef HAVE_INET_ATON
-    unsigned long inaddr;
-#endif /* HAVE_INET_ATON */
-    struct sockaddr_in ad, **pptr;
-    struct hostent *hp;
-
-#ifdef HAVE_SOCKETPAIR
-    if (plugin) {
-      char buf[10];
-#ifdef HAVE_SNPRINTF
-      snprintf(buf, sizeof(buf),  /* Yeah, paranoic. So what? :P */
-#else
-      sprintf(buf,
-#endif /* HAVE_SNPRINTF */
-	      "%d",clientPort);
-      return handle_plugin(host,buf,plugin);
-    }
-#endif /* HAVE_SOCKETPAIR */
-
-    memset(&ad, 0, sizeof(ad));
-    ad.sin_family = AF_INET;
-
-    /* we'll accept a quad address */
-#ifndef HAVE_INET_ATON
-    inaddr = inet_addr((char*)host);
-    if (inaddr != INADDR_NONE)
-    {
-        memcpy(&ad.sin_addr, &inaddr, sizeof(inaddr));
-#else
-    if (inet_aton(host, &ad.sin_addr))
-    {
-#endif /* HAVE_INET_ATON */
-        ad.sin_port = htons(clientPort);
-
-        sock = socket(AF_INET, SOCK_STREAM, 0);
-        if (sock < 0)
-        {
-            h_errno = 0;
-            return -1;
-        }
-
-		/* Socket opened saved. Usefull if connect timeout because
-		 * it can be closed
-		 */
-		mailserver_socket_temp = sock;
-		
-        if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) < 0)
-        {
-            int olderr = errno;
-            fm_close(sock);	/* don't use SockClose, no traffic yet */
-            h_errno = 0;
-            errno = olderr;
-            return -1;
-        }
-
-		/* No connect timeout, then no need to set mailserver_socket_temp */
-		mailserver_socket_temp = -1;
-		
-#ifndef HAVE_INET_ATON
-    }
-#else
-    }
-#endif /* HAVE_INET_ATON */
-    else {
-        hp = gethostbyname((char*)host);
-
-        if (hp == NULL)
-	{
-	    errno = 0;
-	    return -1;
-	}
-	/*
-	 * Add a check to make sure the address has a valid IPv4 or IPv6
-	 * length.  This prevents buffer spamming by a broken DNS.
-	 */
-	if(hp->h_length != 4 && hp->h_length != 8)
-	{
-	    h_errno = errno = 0;
-	    report(stderr, 
-		   GT_("fetchmail: illegal address length received for host %s\n"),host);
-	    return -1;
-	}
-	/*
-	 * Try all addresses of a possibly multihomed host until we get
-	 * a successful connect or until we run out of addresses.
-	 */
-	pptr = (struct sockaddr_in **)hp->h_addr_list;
-	for(; *pptr != NULL; pptr++)
-	{
-	    sock = socket(AF_INET, SOCK_STREAM, 0);
-	    if (sock < 0)
-	    {
-		h_errno = 0;
-		return -1;
-	    }
-
-		/* Socket opened saved. Usefull if connect timeout because
-		 * it can be closed
-		 */
-		mailserver_socket_temp = sock;
-		
-	    ad.sin_port = htons(clientPort);
-	    memcpy(&ad.sin_addr, *pptr, sizeof(struct in_addr));
-	    if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) == 0) {
-			/* No connect timeout, then no need to set mailserver_socket_temp */
-			mailserver_socket_temp = -1;
-			break; /* success */
-		}	
-	    fm_close(sock);	/* don't use SockClose, no traffic yet */
-	    memset(&ad, 0, sizeof(ad));
-	    ad.sin_family = AF_INET;
-	}
-	if(*pptr == NULL)
-	{
-	    int olderr = errno;
-	    fm_close(sock);	/* don't use SockClose, no traffic yet */
-	    h_errno = 0;
-	    errno = olderr;
-	    return -1;
-	}
-    }
-
-    return(sock);
-}
-#endif /* INET6_ENABLE */
 
 
 #if defined(HAVE_STDARG_H)
@@ -490,26 +332,23 @@ va_dcl {
 #else
     va_start(ap);
 #endif
-#ifdef HAVE_VSNPRINTF
     vsnprintf(buf, sizeof(buf), format, ap);
-#else
-    vsprintf(buf, format, ap);
-#endif
     va_end(ap);
     return SockWrite(sock, buf, strlen(buf));
 
 }
 
 #ifdef SSL_ENABLE
-#include "openssl/ssl.h"
-#include "openssl/err.h"
-#include "openssl/pem.h"
-#include "openssl/x509.h"
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include <openssl/pem.h>
+#include <openssl/x509v3.h>
+#include <openssl/rand.h>
 
 static	SSL_CTX *_ctx = NULL;
 static	SSL *_ssl_context[FD_SETSIZE];
 
-SSL	*SSLGetContext( int );
+static SSL	*SSLGetContext( int );
 #endif /* SSL_ENABLE */
 
 int SockWrite(int sock, char *buf, int len)
@@ -540,6 +379,9 @@ int SockRead(int sock, char *buf, int len)
 {
     char *newline, *bp = buf;
     int n;
+#ifdef	FORCE_STUFFING
+    int maxavailable = 0;
+#endif
 #ifdef	SSL_ENABLE
     SSL *ssl;
 #endif
@@ -579,6 +421,9 @@ int SockRead(int sock, char *buf, int len)
 			(void)SSL_get_error(ssl, n);
 			return(-1);
 		}
+#ifdef FORCE_STUFFING
+		maxavailable = n;
+#endif
 		if( 0 == n ) {
 			/* SSL_peek says no data...  Does he mean no data
 			or did the connection blow up?  If we got an error
@@ -622,6 +467,9 @@ int SockRead(int sock, char *buf, int len)
 	    if ((n = fm_peek(sock, bp, len)) <= 0)
 #endif
 		return (-1);
+#ifdef FORCE_STUFFING
+	    maxavailable = n;
+#endif
 	    if ((newline = memchr(bp, '\n', n)) != NULL)
 		n = newline - bp + 1;
 #ifndef __BEOS__
@@ -634,6 +482,47 @@ int SockRead(int sock, char *buf, int len)
     } while 
 	    (!newline && len);
     *bp = '\0';
+
+#ifdef FORCE_STUFFING		/* too ugly to live -- besides, there's IMAP */
+    /* OK, very weird hack coming up here:
+     * When POP3 servers send us a message, they're supposed to
+     * terminate the message with a line containing only a dot. To protect
+     * against lines in the real message that might contain only a dot,
+     * they're supposed to preface any line that starts with a dot with
+     * an additional dot, which will be removed on the client side. That
+     * process, called byte-stuffing (and unstuffing) is really not the
+     * concern of this low-level routine, ordinarily, but there are some
+     * POP servers (and maybe IMAP servers too, who knows) that fail to
+     * do the byte-stuffing, and this routine is the best place to try to
+     * identify and fix that fault.
+     *
+     * Since the DOT line is supposed to come only at the end of a
+     * message, the implication is that right after we see it, the server
+     * is supposed to go back to waiting for the next command. There
+     * isn't supposed to be any more data to read after we see the dot.
+     * THEREFORE, if we see more data to be read after something that
+     * looks like the dot line, then probably the server is failing to
+     * do byte-stuffing. In that case, we'll byte-pack it for them so
+     * that the higher-level routines see things as hunky-dorey.
+     * This is not a perfect test or fix by any means (it has an
+     * obvious race condition, for one thing), but it should at least
+     * reduce the nastiness that ensues when people don't know how
+     * to write POP servers.
+     */
+    if ((maxavailable > (bp-buf)) &&
+	    ((((bp-buf) == 3) &&
+	      (buf[0] == '.') &&
+	      (buf[1] == '\r') &&
+	      (buf[2] == '\n')) ||
+	     (((bp-buf) == 2) &&
+	      (buf[0] == '.') &&
+	      (buf[1] == '\n')))) {
+
+	memmove(buf+1, buf, (bp-buf)+1);
+	buf[0] = '.';
+	bp++;
+    }
+#endif /* FORCE_STUFFING */
     return bp - buf;
 }
 
@@ -694,6 +583,7 @@ static	int _check_fp;
 static	char *_check_digest;
 static 	char *_server_label;
 static	int _depth0ck;
+static	int _prev_err;
 
 SSL *SSLGetContext( int sock )
 {
@@ -707,14 +597,17 @@ SSL *SSLGetContext( int sock )
 }
 
 
-int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
+/* ok_return (preverify_ok) is 1 if this stage of certificate verification
+   passed, or 0 if it failed. This callback lets us display informative
+   errors, and perform additional validation (e.g. CN matches) */
+static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 {
 	char buf[257];
 	X509 *x509_cert;
 	int err, depth;
 	unsigned char digest[EVP_MAX_MD_SIZE];
 	char text[EVP_MAX_MD_SIZE * 3 + 1], *tp, *te;
-	EVP_MD *digest_tp;
+	const EVP_MD *digest_tp;
 	unsigned int dsz, i, esz;
 	X509_NAME *subj, *issuer;
 
@@ -725,10 +618,10 @@ int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 	subj = X509_get_subject_name(x509_cert);
 	issuer = X509_get_issuer_name(x509_cert);
 
-	if (depth == 0) {
+	if (depth == 0 && !_depth0ck) {
 		_depth0ck = 1;
 		
-		if (outlevel == O_VERBOSE) {
+		if (outlevel >= O_VERBOSE) {
 			if ((i = X509_NAME_get_text_by_NID(issuer, NID_organizationName, buf, sizeof(buf))) != -1) {
 				report(stdout, GT_("Issuer Organization: %s\n"), buf);
 				if (i >= sizeof(buf) - 1)
@@ -743,7 +636,7 @@ int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 				report(stdout, GT_("Unknown Issuer CommonName\n"));
 		}
 		if ((i = X509_NAME_get_text_by_NID(subj, NID_commonName, buf, sizeof(buf))) != -1) {
-			if (outlevel == O_VERBOSE)
+			if (outlevel >= O_VERBOSE)
 				report(stdout, GT_("Server CommonName: %s\n"), buf);
 			if (i >= sizeof(buf) - 1) {
 				/* Possible truncation. In this case, this is a DNS name, so this
@@ -755,26 +648,56 @@ int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 				char *p1 = buf;
 				char *p2 = _ssl_server_cname;
 				int n;
+				int matched = 0;
+				STACK_OF(GENERAL_NAME) *gens;
 				
+				/* RFC 2595 section 2.4: find a matching name
+				 * first find a match among alternative names */
+				gens = X509_get_ext_d2i(x509_cert, NID_subject_alt_name, NULL, NULL);
+				if (gens) {
+					int i, r;
+					for (i = 0, r = sk_GENERAL_NAME_num(gens); i < r; ++i) {
+						const GENERAL_NAME *gn = sk_GENERAL_NAME_value(gens, i);
+						if (gn->type == GEN_DNS) {
+							char *p1 = (char *)gn->d.ia5->data;
+							char *p2 = _ssl_server_cname;
+							if (outlevel >= O_VERBOSE)
+								report(stderr, "Subject Alternative Name: %s\n", p1);
+							if (*p1 == '*') {
+								++p1;
+								n = strlen(p2) - strlen(p1);
+								if (n >= 0)
+									p2 += n;
+							}
+							if (0 == strcasecmp(p1, p2)) {
+								matched = 1;
+							}
+						}
+					}
+					sk_GENERAL_NAME_free(gens);
+				}
 				if (*p1 == '*') {
 					++p1;
 					n = strlen(p2) - strlen(p1);
 					if (n >= 0)
 						p2 += n;
 				}	
-				if (0 != strcasecmp(p1, p2)) {
+				if (0 == strcasecmp(p1, p2)) {
+				  matched = 1;
+				}
+				if (!matched) {
 					report(stderr,
 					    GT_("Server CommonName mismatch: %s != %s\n"),
 					    buf, _ssl_server_cname );
 					if (ok_return && strict)
 						return (0);
 				}
-			} else if (ok_return && strict) {
+			} else if (ok_return) {
 				report(stderr, GT_("Server name not set, could not verify certificate!\n"));
-				return (0);
+				if (strict) return (0);
 			}
 		} else {
-			if (outlevel == O_VERBOSE)
+			if (outlevel >= O_VERBOSE)
 				report(stdout, GT_("Unknown Server CommonName\n"));
 			if (ok_return && strict) {
 				report(stderr, GT_("Server name not specified in certificate!\n"));
@@ -797,11 +720,7 @@ int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 			tp = text;
 			te = text + sizeof(text);
 			for (i = 0; i < dsz; i++) {
-#ifdef HAVE_SNPRINTF
 				esz = snprintf(tp, te - tp, i > 0 ? ":%02X" : "%02X", digest[i]);
-#else
-				esz = sprintf(tp, i > 0 ? ":%02X" : "%02X", digest[i]);
-#endif
 				if (esz >= te - tp) {
 					report(stderr, GT_("Digest text buffer too small!\n"));
 					return (0);
@@ -823,28 +742,33 @@ int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 		}
 	}
 
-	if (err != X509_V_OK && (strict || outlevel == O_VERBOSE)) {
-		report(strict ? stderr : stdout, GT_("Warning: server certificate verification: %s\n"), X509_verify_cert_error_string(err));
+	if (err != X509_V_OK && err != _prev_err) {
+        	_prev_err = err;
+		report(stderr, GT_("Server certificate verification error: %s\n"), X509_verify_cert_error_string(err));
 		/* We gave the error code, but maybe we can add some more details for debugging */
 		switch (err) {
 		case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
 			X509_NAME_oneline(issuer, buf, sizeof(buf));
 			buf[sizeof(buf) - 1] = '\0';
-			report(stdout, GT_("unknown issuer (first %d characters): %s\n"), sizeof(buf), buf);
+			report(stderr, GT_("unknown issuer (first %d characters): %s\n"), sizeof(buf)-1, buf);
 			break;
 		}
 	}
+	/*
+	 * If not in strict checking mode (--sslcertck), override this
+	 * and pretend that verification had succeeded.
+	 */
 	if (!strict)
 		ok_return = 1;
 	return (ok_return);
 }
 
-int SSL_nock_verify_callback( int ok_return, X509_STORE_CTX *ctx )
+static int SSL_nock_verify_callback( int ok_return, X509_STORE_CTX *ctx )
 {
 	return SSL_verify_callback(ok_return, ctx, 0);
 }
 
-int SSL_ck_verify_callback( int ok_return, X509_STORE_CTX *ctx )
+static int SSL_ck_verify_callback( int ok_return, X509_STORE_CTX *ctx )
 {
 	return SSL_verify_callback(ok_return, ctx, 1);
 }
@@ -856,11 +780,31 @@ int SSL_ck_verify_callback( int ok_return, X509_STORE_CTX *ctx )
 int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char *certpath,
     char *fingerprint, char *servercname, char *label)
 {
-	SSL *ssl;
-	
+        struct stat randstat;
+        int i;
+
 	SSL_load_error_strings();
 	SSLeay_add_ssl_algorithms();
 	
+#ifdef SSL_ENABLE
+        if (stat("/dev/random", &randstat)  &&
+            stat("/dev/urandom", &randstat)) {
+          /* Neither /dev/random nor /dev/urandom are present, so add
+             entropy to the SSL PRNG a hard way. */
+          for (i = 0; i < 10000  &&  ! RAND_status (); ++i) {
+            char buf[4];
+            struct timeval tv;
+            gettimeofday (&tv, 0);
+            buf[0] = tv.tv_usec & 0xF;
+            buf[2] = (tv.tv_usec & 0xF0) >> 4;
+            buf[3] = (tv.tv_usec & 0xF00) >> 8;
+            buf[1] = (tv.tv_usec & 0xF000) >> 12;
+            RAND_add (buf, sizeof buf, 0.1);
+          }
+        }
+#endif /* SSL_ENABLE */
+
+
 	if( sock < 0 || sock > FD_SETSIZE ) {
 		report(stderr, GT_("File descriptor out of range for SSL") );
 		return( -1 );
@@ -876,6 +820,8 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char
 				_ctx = SSL_CTX_new(SSLv3_client_method());
 			} else if(!strcmp("tls1",myproto)) {
 				_ctx = SSL_CTX_new(TLSv1_client_method());
+			} else if (!strcmp("ssl23",myproto)) {
+				myproto = NULL;
 			} else {
 				fprintf(stderr,GT_("Invalid SSL protocol '%s' specified, using default (SSLv23).\n"), myproto);
 				myproto = NULL;
@@ -892,13 +838,15 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char
 
 	if (certck) {
 		SSL_CTX_set_verify(_ctx, SSL_VERIFY_PEER, SSL_ck_verify_callback);
-		if (certpath)
-			SSL_CTX_load_verify_locations(_ctx, NULL, certpath);
 	} else {
 		/* In this case, we do not fail if verification fails. However,
 		 *  we provide the callback for output and possible fingerprint checks. */
 		SSL_CTX_set_verify(_ctx, SSL_VERIFY_PEER, SSL_nock_verify_callback);
 	}
+	if (certpath)
+		SSL_CTX_load_verify_locations(_ctx, NULL, certpath);
+	else
+		SSL_CTX_set_default_verify_paths(_ctx);
 	
 	_ssl_context[sock] = SSL_new(_ctx);
 	
@@ -913,6 +861,7 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char
 	_check_fp = 1;
 	_check_digest = fingerprint;
 	_depth0ck = 0;
+	_prev_err = -1;
 
 	if( mycert || mykey ) {
 
@@ -930,21 +879,24 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char
 
 	SSL_set_fd(_ssl_context[sock], sock);
 	
-	if(SSL_connect(_ssl_context[sock]) == -1) {
+	if(SSL_connect(_ssl_context[sock]) < 1) {
 		ERR_print_errors_fp(stderr);
 		return(-1);
 	}
 
 	/* Paranoia: was the callback not called as we expected? */
-	if ((fingerprint != NULL || certck) && !_depth0ck) {
+	if (!_depth0ck) {
 		report(stderr, GT_("Certificate/fingerprint verification was somehow skipped!\n"));
-		
-		if( NULL != ( ssl = SSLGetContext( sock ) ) ) {
-			/* Clean up the SSL stack */
-			SSL_free( _ssl_context[sock] );
-			_ssl_context[sock] = NULL;
+
+		if (fingerprint != NULL || certck) {
+			if( NULL != SSLGetContext( sock ) ) {
+				/* Clean up the SSL stack */
+				SSL_shutdown( _ssl_context[sock] );
+				SSL_free( _ssl_context[sock] );
+				_ssl_context[sock] = NULL;
+			}
+			return(-1);
 		}
-		return(-1);
 	}
 
 	return(0);
@@ -955,10 +907,9 @@ int SockClose(int sock)
 /* close a socket gracefully */
 {
 #ifdef	SSL_ENABLE
-    SSL *ssl;
-
-    if( NULL != ( ssl = SSLGetContext( sock ) ) ) {
+    if( NULL != SSLGetContext( sock ) ) {
         /* Clean up the SSL stack */
+        SSL_shutdown( _ssl_context[sock] );
         SSL_free( _ssl_context[sock] );
         _ssl_context[sock] = NULL;
     }
@@ -1016,6 +967,8 @@ static ssize_t cygwin_read(int sock, void *buf, size_t count)
 	    return(-1);
 	}
     }
+
+    return count;
 }
 #endif /* __CYGWIN__ */
 
@@ -1026,7 +979,7 @@ static ssize_t cygwin_read(int sock, void *buf, size_t count)
  * inetd.conf (and then SIGHUP inetd) for this to work.  */
 main()
 {
-    int	 	sock = SockOpen("localhost", 19, NULL);
+    int	 	sock = SockOpen("localhost", "chargen", NULL);
     char	buf[80];
 
     while (SockRead(sock, buf, sizeof(buf)-1))
